@@ -21,7 +21,6 @@ import PIL
 
 from paddlenlp.transformers import CLIPFeatureExtractor, CLIPTokenizer
 
-from ...fastdeploy_utils import FastDeployRuntimeModel
 from ...pipeline_utils import DiffusionPipeline
 from ...schedulers import (
     DDIMScheduler,
@@ -32,6 +31,7 @@ from ...schedulers import (
     PNDMScheduler,
 )
 from ...utils import PIL_INTERPOLATION, logging
+from ..fastdeploy_utils import FastDeployRuntimeModel
 from . import StableDiffusionPipelineOutput
 
 logger = logging.get_logger(__name__)  # pylint: disable=invalid-name
@@ -121,7 +121,7 @@ class FastDeployStableDiffusionInpaintPipeline(DiffusionPipeline):
             )
         if safety_checker is not None and feature_extractor is None:
             raise ValueError(
-                "Make sure to define a feature extractor when loading {self.__class__} if you want to use the safety"
+                f"Make sure to define a feature extractor when loading {self.__class__} if you want to use the safety"
                 " checker. If you do not want to use the safety checker, you can pass `'safety_checker=None'` instead."
             )
 
@@ -275,7 +275,7 @@ class FastDeployStableDiffusionInpaintPipeline(DiffusionPipeline):
 
         latents_shape = (batch_size, num_channels_latents, height // 8, width // 8)
         if latents is None:
-            latents = generator.randn(*latents_shape).astype(dtype)
+            latents = paddle.to_tensor(generator.randn(*latents_shape), dtype=dtype)
         elif latents.shape != latents_shape:
             raise ValueError(f"Unexpected latents shape, got {latents.shape}, expected {latents_shape}")
 
@@ -399,7 +399,7 @@ class FastDeployStableDiffusionInpaintPipeline(DiffusionPipeline):
 
         # 4. set timesteps
         self.scheduler.set_timesteps(num_inference_steps)
-        timesteps = self.scheduler.timesteps.numpy()
+        timesteps = self.scheduler.timesteps
 
         # 5. Prepare latent variables
         num_channels_latents = NUM_LATENT_CHANNELS
@@ -427,6 +427,8 @@ class FastDeployStableDiffusionInpaintPipeline(DiffusionPipeline):
         )
         num_channels_mask = mask.shape[1]
         num_channels_masked_image = masked_image_latents.shape[1]
+        mask = paddle.to_tensor(mask)
+        masked_image_latents = paddle.to_tensor(masked_image_latents)
 
         # 8. Check that sizes of mask, masked image and latents match
         unet_input_channels = NUM_UNET_INPUT_CHANNELS
@@ -441,36 +443,33 @@ class FastDeployStableDiffusionInpaintPipeline(DiffusionPipeline):
 
         # 9. Prepare extra step kwargs.
         extra_step_kwargs = self.prepare_extra_step_kwargs(eta)
+        text_embeddings = paddle.to_tensor(text_embeddings, dtype="float32")
 
         # 10. Denoising loop
         num_warmup_steps = len(timesteps) - num_inference_steps * self.scheduler.order
         with self.progress_bar(total=num_inference_steps) as progress_bar:
             for i, t in enumerate(timesteps):
-                tensor_t = paddle.to_tensor(t)
                 # expand the latents if we are doing classifier free guidance
-                latent_model_input = np.concatenate([latents] * 2) if do_classifier_free_guidance else latents
+                latent_model_input = paddle.concat([latents] * 2) if do_classifier_free_guidance else latents
                 # concat latents, mask, masked_image_latnets in the channel dimension
-                latent_model_input = self.scheduler.scale_model_input(paddle.to_tensor(latent_model_input), tensor_t)
-                latent_model_input = latent_model_input.numpy()
-                latent_model_input = np.concatenate([latent_model_input, mask, masked_image_latents], axis=1)
+                latent_model_input = self.scheduler.scale_model_input(latent_model_input, t)
+                latent_model_input = paddle.concat([latent_model_input, mask, masked_image_latents], axis=1)
 
                 # predict the noise residual
-                noise_pred = self.unet(
-                    sample=latent_model_input.astype(np.float32),
-                    timestep=np.array([t], dtype=np.int64),
-                    encoder_hidden_states=text_embeddings.astype(np.float32),
+                noise_pred_unet = self.unet(
+                    sample=latent_model_input.numpy(),
+                    timestep=t.cast("float32").numpy(),
+                    encoder_hidden_states=text_embeddings.numpy(),
                 )[0]
-
+                noise_pred_unet = paddle.to_tensor(noise_pred_unet)
                 # perform guidance
                 if do_classifier_free_guidance:
-                    noise_pred_uncond, noise_pred_text = np.split(noise_pred, 2)
+                    noise_pred_uncond, noise_pred_text = noise_pred_unet.chunk(2)
                     noise_pred = noise_pred_uncond + guidance_scale * (noise_pred_text - noise_pred_uncond)
 
                 # compute the previous noisy sample x_t -> x_t-1
-                scheduler_output = self.scheduler.step(
-                    paddle.to_tensor(noise_pred), tensor_t, paddle.to_tensor(latents), **extra_step_kwargs
-                )
-                latents = scheduler_output.prev_sample.numpy()
+                scheduler_output = self.scheduler.step(noise_pred, t, latents, **extra_step_kwargs)
+                latents = scheduler_output.prev_sample
 
                 # call the callback, if provided
                 if i == len(timesteps) - 1 or ((i + 1) > num_warmup_steps and (i + 1) % self.scheduler.order == 0):
@@ -479,7 +478,7 @@ class FastDeployStableDiffusionInpaintPipeline(DiffusionPipeline):
                         callback(i, t, latents)
 
         # 11. Post-processing
-        image = self.decode_latents(latents)
+        image = self.decode_latents(latents.numpy())
 
         # 12. Run safety checker
         image, has_nsfw_concept = self.run_safety_checker(image, text_embeddings.dtype)
