@@ -1,5 +1,5 @@
-# coding=utf-8
-# Copyright 2022 HuggingFace Inc.
+# Copyright (c) 2023 PaddlePaddle Authors. All Rights Reserved.
+# Copyright 2023 The HuggingFace Team. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,10 +20,14 @@ import unittest
 import numpy as np
 import paddle
 from PIL import Image
-from test_pipelines_common import PipelineTesterMixin
+from ppdiffusers_test.pipeline_params import (
+    IMAGE_VARIATION_BATCH_PARAMS,
+    IMAGE_VARIATION_PARAMS,
+)
+from ppdiffusers_test.test_pipelines_common import PipelineTesterMixin
 
 from paddlenlp.transformers import (
-    CLIPFeatureExtractor,
+    CLIPImageProcessor,
     CLIPVisionConfig,
     CLIPVisionModelWithProjection,
 )
@@ -34,18 +38,14 @@ from ppdiffusers import (
     StableDiffusionImageVariationPipeline,
     UNet2DConditionModel,
 )
-from ppdiffusers.utils import (
-    TEST_DOWNLOAD_SERVER,
-    floats_tensor,
-    load_image,
-    load_numpy,
-    nightly,
-    slow,
-)
+from ppdiffusers.utils import floats_tensor, load_image, load_numpy, nightly, slow
+from ppdiffusers.utils.testing_utils import require_paddle_gpu
 
 
 class StableDiffusionImageVariationPipelineFastTests(PipelineTesterMixin, unittest.TestCase):
     pipeline_class = StableDiffusionImageVariationPipeline
+    params = IMAGE_VARIATION_PARAMS
+    batch_params = IMAGE_VARIATION_BATCH_PARAMS
 
     def get_dummy_components(self):
         paddle.seed(0)
@@ -70,20 +70,18 @@ class StableDiffusionImageVariationPipelineFastTests(PipelineTesterMixin, unitte
             latent_channels=4,
         )
         paddle.seed(0)
-        image_encoder_config = dict(
-            vision_embed_dim=32,
+        image_encoder_config = CLIPVisionConfig(
+            hidden_size=32,
             projection_dim=32,
-            vision_mlp_ratio=1,
-            vision_heads=4,
-            vision_layers=5,
-            image_resolution=32,
-            vision_patch_size=4,
+            intermediate_size=37,
+            layer_norm_eps=1e-05,
+            num_attention_heads=4,
+            num_hidden_layers=5,
+            image_size=32,
+            patch_size=4,
         )
-        image_encoder_config = CLIPVisionConfig.from_dict(image_encoder_config)
         image_encoder = CLIPVisionModelWithProjection(image_encoder_config)
-        image_encoder.eval()
-        feature_extractor = CLIPFeatureExtractor(crop_size=32)
-
+        feature_extractor = CLIPImageProcessor(crop_size=32, size=32)
         components = {
             "unet": unet,
             "scheduler": scheduler,
@@ -96,9 +94,10 @@ class StableDiffusionImageVariationPipelineFastTests(PipelineTesterMixin, unitte
 
     def get_dummy_inputs(self, seed=0):
         image = floats_tensor((1, 3, 32, 32), rng=random.Random(seed))
-        image = image.cpu().transpose([0, 2, 3, 1])[0]
+        image = image.cpu().transpose(perm=[0, 2, 3, 1])[0]
         image = Image.fromarray(np.uint8(image)).convert("RGB").resize((32, 32))
         generator = paddle.Generator().manual_seed(seed)
+
         inputs = {
             "image": image,
             "generator": generator,
@@ -112,105 +111,69 @@ class StableDiffusionImageVariationPipelineFastTests(PipelineTesterMixin, unitte
         components = self.get_dummy_components()
         sd_pipe = StableDiffusionImageVariationPipeline(**components)
         sd_pipe.set_progress_bar_config(disable=None)
-
         inputs = self.get_dummy_inputs()
         image = sd_pipe(**inputs).images
-
         image_slice = image[0, -3:, -3:, -1]
-
         assert image.shape == (1, 64, 64, 3)
         expected_slice = np.array(
-            [
-                0.3151392638683319,
-                0.11757123470306396,
-                0.30373042821884155,
-                0.20310524106025696,
-                0.20141255855560303,
-                0.5957024097442627,
-                0.25098103284835815,
-                0.23880955576896667,
-                0.5245755910873413,
-            ]
+            [0.3336423, 0.15990603, 0.33051074, 0.17818755, 0.22190896, 0.580778, 0.23555121, 0.2542741, 0.5196996]
         )
-        assert np.abs(image_slice.flatten() - expected_slice).max() < 1e-3
+
+        assert np.abs(image_slice.flatten() - expected_slice).max() < 0.001
 
     def test_stable_diffusion_img_variation_multiple_images(self):
         components = self.get_dummy_components()
         sd_pipe = StableDiffusionImageVariationPipeline(**components)
-
         sd_pipe.set_progress_bar_config(disable=None)
-
         inputs = self.get_dummy_inputs()
         inputs["image"] = 2 * [inputs["image"]]
         output = sd_pipe(**inputs)
-
         image = output.images
-
         image_slice = image[-1, -3:, -3:, -1]
-
         assert image.shape == (2, 64, 64, 3)
         expected_slice = np.array(
-            [
-                0.6137708425521851,
-                0.6509915590286255,
-                0.6715413331985474,
-                0.522708535194397,
-                0.31863510608673096,
-                0.46015962958335876,
-                0.5901781916618347,
-                0.3334598243236542,
-                0.41584062576293945,
-            ]
+            [0.5974754, 0.6734819, 0.67569935, 0.5421418, 0.34375697, 0.46754372, 0.5809557, 0.32910222, 0.41817445]
         )
-        assert np.abs(image_slice.flatten() - expected_slice).max() < 1e-3
+        assert np.abs(image_slice.flatten() - expected_slice).max() < 0.001
 
     def test_stable_diffusion_img_variation_num_images_per_prompt(self):
         components = self.get_dummy_components()
         sd_pipe = StableDiffusionImageVariationPipeline(**components)
         sd_pipe.set_progress_bar_config(disable=None)
-
-        # test num_images_per_prompt=1 (default)
         inputs = self.get_dummy_inputs()
         images = sd_pipe(**inputs).images
-
         assert images.shape == (1, 64, 64, 3)
-
-        # test num_images_per_prompt=1 (default) for batch of images
         batch_size = 2
         inputs = self.get_dummy_inputs()
         inputs["image"] = batch_size * [inputs["image"]]
         images = sd_pipe(**inputs).images
-
         assert images.shape == (batch_size, 64, 64, 3)
-
-        # test num_images_per_prompt for single prompt
         num_images_per_prompt = 2
         inputs = self.get_dummy_inputs()
         images = sd_pipe(**inputs, num_images_per_prompt=num_images_per_prompt).images
-
         assert images.shape == (num_images_per_prompt, 64, 64, 3)
-
-        # test num_images_per_prompt for batch of prompts
         batch_size = 2
         inputs = self.get_dummy_inputs()
         inputs["image"] = batch_size * [inputs["image"]]
         images = sd_pipe(**inputs, num_images_per_prompt=num_images_per_prompt).images
-
         assert images.shape == (batch_size * num_images_per_prompt, 64, 64, 3)
 
 
 @slow
+@require_paddle_gpu
 class StableDiffusionImageVariationPipelineSlowTests(unittest.TestCase):
     def tearDown(self):
         super().tearDown()
         gc.collect()
         paddle.device.cuda.empty_cache()
 
-    def get_inputs(self, dtype=paddle.float32, seed=0):
+    def get_inputs(self, dtype="float32", seed=0):
         generator = paddle.Generator().manual_seed(seed)
-        init_image = load_image(f"{TEST_DOWNLOAD_SERVER}/stable_diffusion_imgvar/input_image_vermeer.png")
+        init_image = load_image(
+            "https://huggingface.co/datasets/diffusers/test-arrays/resolve/main/stable_diffusion_imgvar/input_image_vermeer.png"
+        )
         latents = np.random.RandomState(seed).standard_normal((1, 4, 64, 64))
-        latents = paddle.to_tensor(latents, dtype=dtype)
+        latents = paddle.to_tensor(latents).cast(dtype)
         inputs = {
             "image": init_image,
             "latents": latents,
@@ -221,17 +184,14 @@ class StableDiffusionImageVariationPipelineSlowTests(unittest.TestCase):
         }
         return inputs
 
-    # lambdalabs/sd-image-variations-diffusers
     def test_stable_diffusion_img_variation_pipeline_default(self):
         sd_pipe = StableDiffusionImageVariationPipeline.from_pretrained(
             "fusing/sd-image-variations-diffusers", safety_checker=None
         )
         sd_pipe.set_progress_bar_config(disable=None)
-
         inputs = self.get_inputs()
         image = sd_pipe(**inputs).images
         image_slice = image[0, -3:, -3:, -1].flatten()
-
         assert image.shape == (1, 512, 512, 3)
         expected_slice = np.array(
             [
@@ -246,7 +206,7 @@ class StableDiffusionImageVariationPipelineSlowTests(unittest.TestCase):
                 0.5223015546798706,
             ]
         )
-        assert np.abs(image_slice - expected_slice).max() < 1e-4
+        assert np.abs(image_slice - expected_slice).max() < 0.0001
 
     def test_stable_diffusion_img_variation_intermediate_state(self):
         number_of_steps = 0
@@ -260,65 +220,43 @@ class StableDiffusionImageVariationPipelineSlowTests(unittest.TestCase):
                 assert latents.shape == (1, 4, 64, 64)
                 latents_slice = latents[0, -3:, -3:, -1]
                 expected_slice = np.array(
-                    [
-                        -0.15777504444122314,
-                        0.2840809226036072,
-                        -0.8084282875061035,
-                        -0.11453461647033691,
-                        -1.3060585260391235,
-                        0.7711117267608643,
-                        -2.119102954864502,
-                        0.041628703474998474,
-                        1.6307991743087769,
-                    ]
+                    [-0.1621, 0.2837, -0.7979, -0.1221, -1.3057, 0.7681, -2.1191, 0.0464, 1.6309]
                 )
-                assert np.abs(latents_slice.flatten() - expected_slice).max() < 5e-3
+                assert np.abs(latents_slice.flatten() - expected_slice).max() < 0.05
             elif step == 2:
                 latents = latents.detach().cpu().numpy()
                 assert latents.shape == (1, 4, 64, 64)
                 latents_slice = latents[0, -3:, -3:, -1]
-                expected_slice = np.array(
-                    [
-                        0.5763851404190063,
-                        1.703276515007019,
-                        1.1998151540756226,
-                        -2.148696184158325,
-                        -1.9190490245819092,
-                        0.6978652477264404,
-                        -0.695953369140625,
-                        1.0288335084915161,
-                        1.5673096179962158,
-                    ]
-                )
-                assert np.abs(latents_slice.flatten() - expected_slice).max() < 5e-2
+                expected_slice = np.array([0.6299, 1.75, 1.1992, -2.1582, -1.8994, 0.7334, -0.709, 1.0137, 1.5273])
+                assert np.abs(latents_slice.flatten() - expected_slice).max() < 0.05
 
         callback_fn.has_been_called = False
-
         pipe = StableDiffusionImageVariationPipeline.from_pretrained(
-            "fusing/sd-image-variations-diffusers",
-            safety_checker=None,
+            "fusing/sd-image-variations-diffusers", safety_checker=None, paddle_dtype=paddle.float16
         )
         pipe.set_progress_bar_config(disable=None)
         pipe.enable_attention_slicing()
-
-        inputs = self.get_inputs()
+        inputs = self.get_inputs(dtype="float16")
         pipe(**inputs, callback=callback_fn, callback_steps=1)
         assert callback_fn.has_been_called
         assert number_of_steps == inputs["num_inference_steps"]
 
 
 @nightly
+@require_paddle_gpu
 class StableDiffusionImageVariationPipelineNightlyTests(unittest.TestCase):
     def tearDown(self):
         super().tearDown()
         gc.collect()
         paddle.device.cuda.empty_cache()
 
-    def get_inputs(self, dtype=paddle.float32, seed=0):
+    def get_inputs(self, dtype="float32", seed=0):
         generator = paddle.Generator().manual_seed(seed)
-        init_image = load_image(f"{TEST_DOWNLOAD_SERVER}/stable_diffusion_imgvar/input_image_vermeer.png")
+        init_image = load_image(
+            "https://huggingface.co/datasets/diffusers/test-arrays/resolve/main/stable_diffusion_imgvar/input_image_vermeer.png"
+        )
         latents = np.random.RandomState(seed).standard_normal((1, 4, 64, 64))
-        latents = paddle.to_tensor(latents, dtype=dtype)
+        latents = paddle.to_tensor(latents).cast(dtype)
         inputs = {
             "image": init_image,
             "latents": latents,
@@ -332,25 +270,23 @@ class StableDiffusionImageVariationPipelineNightlyTests(unittest.TestCase):
     def test_img_variation_pndm(self):
         sd_pipe = StableDiffusionImageVariationPipeline.from_pretrained("fusing/sd-image-variations-diffusers")
         sd_pipe.set_progress_bar_config(disable=None)
-
         inputs = self.get_inputs()
         image = sd_pipe(**inputs).images[0]
-
-        expected_image = load_numpy(f"{TEST_DOWNLOAD_SERVER}/stable_diffusion_imgvar/lambdalabs_variations_pndm.npy")
+        expected_image = load_numpy(
+            "https://huggingface.co/datasets/diffusers/test-arrays/resolve/main/stable_diffusion_imgvar/lambdalabs_variations_pndm.npy"
+        )
         max_diff = np.abs(expected_image - image).max()
-        assert max_diff < 1e-3
+        assert max_diff < 0.001
 
     def test_img_variation_dpm(self):
         sd_pipe = StableDiffusionImageVariationPipeline.from_pretrained("fusing/sd-image-variations-diffusers")
         sd_pipe.scheduler = DPMSolverMultistepScheduler.from_config(sd_pipe.scheduler.config)
         sd_pipe.set_progress_bar_config(disable=None)
-
         inputs = self.get_inputs()
         inputs["num_inference_steps"] = 25
         image = sd_pipe(**inputs).images[0]
-
         expected_image = load_numpy(
-            f"{TEST_DOWNLOAD_SERVER}/stable_diffusion_imgvar/lambdalabs_variations_dpm_multi.npy"
+            "https://huggingface.co/datasets/diffusers/test-arrays/resolve/main/stable_diffusion_imgvar/lambdalabs_variations_dpm_multi.npy"
         )
         max_diff = np.abs(expected_image - image).max()
-        assert max_diff < 1e-3
+        assert max_diff < 0.001
