@@ -1,5 +1,5 @@
-# coding=utf-8
-# Copyright 2022 HuggingFace Inc.
+# Copyright (c) 2023 PaddlePaddle Authors. All Rights Reserved.
+# Copyright 2023 The HuggingFace Team. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,7 +21,7 @@ import numpy as np
 import paddle
 
 from ppdiffusers import VersatileDiffusionDualGuidedPipeline
-from ppdiffusers.utils.testing_utils import load_image, slow
+from ppdiffusers.utils.testing_utils import load_image, require_paddle_gpu, slow
 
 
 class VersatileDiffusionDualGuidedPipelineFastTests(unittest.TestCase):
@@ -29,21 +29,20 @@ class VersatileDiffusionDualGuidedPipelineFastTests(unittest.TestCase):
 
 
 @slow
+@require_paddle_gpu
 class VersatileDiffusionDualGuidedPipelineIntegrationTests(unittest.TestCase):
     def tearDown(self):
-        # clean up the VRAM after each test
         super().tearDown()
         gc.collect()
         paddle.device.cuda.empty_cache()
 
     def test_remove_unused_weights_save_load(self):
         pipe = VersatileDiffusionDualGuidedPipeline.from_pretrained("shi-labs/versatile-diffusion")
-        # remove text_unet
         pipe.remove_unused_weights()
         pipe.set_progress_bar_config(disable=None)
-
-        second_prompt = load_image("https://paddlenlp.bj.bcebos.com/models/community/CompVis/data/benz.jpg")
-
+        second_prompt = load_image(
+            "https://huggingface.co/datasets/hf-internal-testing/diffusers-images/resolve/main/versatile_diffusion/benz.jpg"
+        )
         generator = paddle.Generator().manual_seed(0)
         image = pipe(
             prompt="first prompt",
@@ -54,13 +53,10 @@ class VersatileDiffusionDualGuidedPipelineIntegrationTests(unittest.TestCase):
             num_inference_steps=2,
             output_type="numpy",
         ).images
-
         with tempfile.TemporaryDirectory() as tmpdirname:
             pipe.save_pretrained(tmpdirname)
-            pipe = VersatileDiffusionDualGuidedPipeline.from_pretrained(tmpdirname)
-
+            pipe = VersatileDiffusionDualGuidedPipeline.from_pretrained(tmpdirname, from_diffusers=False)
         pipe.set_progress_bar_config(disable=None)
-
         generator = paddle.Generator().manual_seed(0)
         new_image = pipe(
             prompt="first prompt",
@@ -71,16 +67,16 @@ class VersatileDiffusionDualGuidedPipelineIntegrationTests(unittest.TestCase):
             num_inference_steps=2,
             output_type="numpy",
         ).images
-
-        assert np.abs(image - new_image).sum() < 1e-5, "Models don't have the same forward pass"
+        assert np.abs(image - new_image).sum() < 1e-05, "Models don't have the same forward pass"
 
     def test_inference_dual_guided(self):
         pipe = VersatileDiffusionDualGuidedPipeline.from_pretrained("shi-labs/versatile-diffusion")
         pipe.remove_unused_weights()
         pipe.set_progress_bar_config(disable=None)
-
         first_prompt = "cyberpunk 2077"
-        second_prompt = load_image("https://paddlenlp.bj.bcebos.com/models/community/CompVis/data/benz.jpg")
+        second_prompt = load_image(
+            "https://huggingface.co/datasets/hf-internal-testing/diffusers-images/resolve/main/versatile_diffusion/benz.jpg"
+        )
         generator = paddle.Generator().manual_seed(0)
         image = pipe(
             prompt=first_prompt,
@@ -91,21 +87,19 @@ class VersatileDiffusionDualGuidedPipelineIntegrationTests(unittest.TestCase):
             num_inference_steps=50,
             output_type="numpy",
         ).images
-
         image_slice = image[0, 253:256, 253:256, -1]
-
         assert image.shape == (1, 512, 512, 3)
         expected_slice = np.array(
             [
-                0.05819129943847656,
-                0.06465867161750793,
-                0.0698845386505127,
-                0.06031566858291626,
-                0.0696127712726593,
-                0.07399758696556091,
-                0.06906205415725708,
-                0.07634878158569336,
-                0.0813780426979065,
+                0.01500076,
+                0.01142624,
+                0.01418972,
+                0.01518875,
+                0.01114869,
+                0.01190853,
+                0.02978998,
+                0.02376354,
+                0.02396089,
             ]
         )
-        assert np.abs(image_slice.flatten() - expected_slice).max() < 1e-2
+        assert np.abs(image_slice.flatten() - expected_slice).max() < 0.01
